' ********** Copyright 2016 Roku Corp.  All Rights Reserved. **********

'A full RAF integration Example:
' - Include RAF.
' - setAdURL to set the ad URL.
' - Examples of RAF MACROS being passed in the ad call.
' - getAds() for VAST parsing.
' - showAds for rendering.
' - Enable Nielsen.
' - Pass all parameters to Nielsen beacons with examples of genre, program id and content.
'@param videoContent [AA] object that has valid data for playing video with roVideoScreen.
Sub CustomAdBufferingScreen(videoContent as Object)
    'Main facade creation.
    canvas = CreateObject("roImageCanvas")
    canvas.SetLayer(1, {color: "#000000"})
    canvas.SetLayer(2, {text: "Loading..."})
    canvas.Show()

    adIface = Roku_Ads() 'RAF initialize
    print "Roku_Ads library version: " + adIface.getLibVersion()

    ' Content Meta-Data objects which represent layers of roImageCanvas
    layer_0 = {
        Url: "https://upload.wikimedia.org/wikipedia/commons/thumb/f/f8/Aspect-ratio-16x9.svg/1280px-Aspect-ratio-16x9.svg.png"
    }
    layer_1 = {
        Url: "http://static.commentcamarche.net/ccm.net/faq/images/0-BX4VeV6H-resolution-comparison-s-.png"
        TargetRect : {x : 405, y : 370, w : 467, h : 262}
    }
    layer_2 = {
        Text : "This is a custom build screen"
        TextAttrs : { Color : "#FF0000", HAlign : "Center", Font : "Large"}
        TargetRect : {y : 50, h : 30}
    }

    ' Set custom layers with specified zOrder
    adIface.setAdBufferScreenLayer(0, layer_0)
    adIface.setAdBufferScreenLayer(1, layer_1)
    adIface.setAdBufferScreenLayer(2, layer_2)

    ' Set callback function for handling ad buffering events
    adIface.setAdBufferRenderCallback(adBufferingCallback)

    adIface.setDebugOutput(true) 'for debug pupropse

    'RAF content params
    adIface.setContentId(videoContent.contentId)
    adIface.SetContentGenre(videoContent.contentGenre)
    
    'Nielsen content params
    adIface.enableNielsenDAR(true)
    adIface.setContentLength(videoContent.conntentLength)
    adIface.setNielsenProgramId(videoContent.nielsenProgramId)
    adIface.setNielsenGenre(videoContent.nielsenGenre)
    adIface.setNielsenAppId(videoContent.nielsenAppId)

    'Indicates whether the default Roku backfill ad service URL 
    'should be used in case the client-configured URL fails (2 retries)
    'to return any renderable ads.
    adIface.setAdPrefs(true, 2)
   
    'Normally, would set publisher's ad URL here.
    'Otherwise uses default Roku ad server (with single preroll placeholder ad)
    adIface.setAdUrl(videoContent.adUrl) 

    'Returns available ad pod(s) scheduled for rendering or invalid, if none are available.
    adPods = adIface.getAds()
    playContent = adIface.showAds(adPods) 'show preroll ad pod (if any)

    curPos = 0
    if playContent
        videoScreen = PlayVideoContent(videoContent)
    end if

    closingContentScreen = false
    contentDone = false
    while playContent 
        videoMsg = wait(0, videoScreen.GetMessagePort())
        if type(videoMsg) = "roVideoScreenEvent"
            if videoMsg.isStreamStarted()
                canvas.ClearLayer(2)
            end if
            if videoMsg.isPlaybackPosition()
                'cache current playback position for resume after midroll ads
                curPos = videoMsg.GetIndex()
            end if

            if not closingContentScreen 'don't check for any more ads while waiting for screen close
                if videoMsg.isScreenClosed() 'roVideoScreen sends this message last for all exit conditions
                    playContent = false
               else if videoMsg.isFullResult()
                    contentDone = true 'don't want to resume playback after postroll ads
               end if

               'check for midroll/postroll ad pods
               adPods = adIface.getAds(videoMsg)
               if adPods <> invalid and adPods.Count() > 0
                   'must completely close content screen before showing ads
                   'for some Roku platforms (e.g., RokuTV), calling Close() will not synchronously
                   'close the media player and may prevent a new media player from being created
                   'until the screen is fully closed (app has received the isScreenClosed() event)
                   videoScreen.Close()
                   closingContentScreen = true
               end if
            else if videoMsg.isScreenClosed()
                closingContentScreen = false 'now safe to render ads
            end if 'closingContentScreen

            if not closingContentScreen and adPods <> invalid and adPods.Count() > 0
                'now safe to render midroll/postroll ads
                playContent = adIface.showAds(adPods)
                playContent = playContent and not contentDone
                if playContent
                    'resume video playback after midroll ads
                    videoContent.PlayStart = curPos
                    videoScreen = PlayVideoContent(videoContent)
                end if
            end if '!closingContentScreen
        end if 'roVideoScreenEvent
    end while
    if type(videoScreen) = "roVideoScreen" then videoScreen.Close()
End Sub

'*********************************************************************
'** adBufferingCallback
'** @brief Callback function for hadling ad buffering events
'**
'** @param adBufferingCallbackObj - Object passed by client to be received in callback
'** @param eventType - Type of event to trigger callback
'** @param ctx - Structure to capture context-specific trigger conditions
'** { canvasLayers: roArray or roAssociativeArrays passed in SetBufferScreenLayer, invalid if no content
'**   progress    : Int (progress percentage 0 - 100) [optional, only for "Progress" events"]
'**   ad          : ad metadata,
'**   adIndex     : current ad index
'** }
'*********************************************************************
Sub adBufferingCallback(adBufferingCallbackObj, eventType, ctx)
    ' Handling "Progress" event. 
    if eventType = "Progress" and ctx.progress <> invalid then
        
        ' Compiling of progress message to represent progress percentage
        progressText = "This is a custom progress - " + str(ctx.progress) + " % loaded"
        
        ' Adding additional layer to represent progress message
        progressTextLayer = {
            Text : progressText
            TextAttrs : { Color : "#00FF00", HAlign : "Center", Font : "Large"}
            TargetRect : {y : 150, h : 30}
        }
        ctx.canvasLayers[3] = progressTextLayer
    end if
End Sub