' ********** Copyright 2016 Roku Corp.  All Rights Reserved. **********

'A full RAF integration Example:
' - Include RAF.
' - setAdURL to set the ad URL.
' - Examples of RAF MACROS being passed in the ad call.
' - getAds() for VAST parsing.
' - showAds for rendering.
' - Enable Nielsen.
' - Pass all parameters to Nielsen beacons with examples of genre, program id and content.
'@param videoContent [AA] object that has valid data for playing video with roVideoScreen.
Sub CustomizedAdBufferingScreen(videoContent as Object)
    'Main facade creation.
    canvas = CreateObject("roImageCanvas")
    canvas.SetLayer(1, {color: "#000000"})
    canvas.SetLayer(2, {text: "Loading..."})
    canvas.Show()

    adIface = Roku_Ads() 'RAF initialize
    print "Roku_Ads library version: " + adIface.getLibVersion()

    'Standard content-meta-data structure representing the content that will be used to populate the buffer screen
    bufferScreenContent = {
        'Optional Image for background in HD mode 1280*720
        HDBackgroundImageUrl : "https://upload.wikimedia.org/wikipedia/commons/thumb/f/f8/Aspect-ratio-16x9.svg/1280px-Aspect-ratio-16x9.svg.png"
        
        'Optional Image for background in SD mode 720*480
        SDBackgroundImageUrl : "https://upload.wikimedia.org/wikipedia/commons/thumb/f/f8/Aspect-ratio-16x9.svg/1280px-Aspect-ratio-16x9.svg.png"
        
        'Optional centered HD image (content poster/logo/etc.) 467*262
        HDPosterUrl : "http://static.commentcamarche.net/ccm.net/faq/images/0-BX4VeV6H-resolution-comparison-s-.png"
        
        'Optional centered SD image (content poster/logo/etc.) 312*175
        SDPosterUrl : "http://static.commentcamarche.net/ccm.net/faq/images/0-BX4VeV6H-resolution-comparison-s-.png"
        
        'Optional Title of content
        Title : "Title title title"
        
        'Optional Description of the content
        Description : "Description description description description description description description description description description description description description description description description description"
    }

    ' Set content-meta-data structure to customize default ad buffering screen
    adIface.SetAdBufferScreenContent(bufferScreenContent)

    adIface.setDebugOutput(true) 'for debug pupropse

    'RAF content params
    adIface.setContentId(videoContent.contentId)
    adIface.SetContentGenre(videoContent.contentGenre)
    
    'Nielsen content params
    adIface.enableNielsenDAR(true)
    adIface.setContentLength(videoContent.conntentLength)
    adIface.setNielsenProgramId(videoContent.nielsenProgramId)
    adIface.setNielsenGenre(videoContent.nielsenGenre)
    adIface.setNielsenAppId(videoContent.nielsenAppId)

    'Indicates whether the default Roku backfill ad service URL 
    'should be used in case the client-configured URL fails (2 retries)
    'to return any renderable ads.
    adIface.setAdPrefs(true, 2)
   
    'Normally, would set publisher's ad URL here.
    'Otherwise uses default Roku ad server (with single preroll placeholder ad)
    adIface.setAdUrl(videoContent.adUrl) 

    'Returns available ad pod(s) scheduled for rendering or invalid, if none are available.
    adPods = adIface.getAds()
    playContent = adIface.showAds(adPods) 'show preroll ad pod (if any)

    curPos = 0
    if playContent
        videoScreen = PlayVideoContent(videoContent)
    end if

    closingContentScreen = false
    contentDone = false
    while playContent 
        videoMsg = wait(0, videoScreen.GetMessagePort())
        if type(videoMsg) = "roVideoScreenEvent"
            if videoMsg.isStreamStarted()
                canvas.ClearLayer(2)
            end if
            if videoMsg.isPlaybackPosition()
                'cache current playback position for resume after midroll ads
                curPos = videoMsg.GetIndex()
            end if

            if not closingContentScreen 'don't check for any more ads while waiting for screen close
                if videoMsg.isScreenClosed() 'roVideoScreen sends this message last for all exit conditions
                    playContent = false
               else if videoMsg.isFullResult()
                    contentDone = true 'don't want to resume playback after postroll ads
               end if

               'check for midroll/postroll ad pods
               adPods = adIface.getAds(videoMsg)
               if adPods <> invalid and adPods.Count() > 0
                   'must completely close content screen before showing ads
                   'for some Roku platforms (e.g., RokuTV), calling Close() will not synchronously
                   'close the media player and may prevent a new media player from being created
                   'until the screen is fully closed (app has received the isScreenClosed() event)
                   videoScreen.Close()
                   closingContentScreen = true
               end if
            else if videoMsg.isScreenClosed()
                closingContentScreen = false 'now safe to render ads
            end if 'closingContentScreen

            if not closingContentScreen and adPods <> invalid and adPods.Count() > 0
                'now safe to render midroll/postroll ads
                playContent = adIface.showAds(adPods)
                playContent = playContent and not contentDone
                if playContent
                    'resume video playback after midroll ads
                    videoContent.PlayStart = curPos
                    videoScreen = PlayVideoContent(videoContent)
                end if
            end if '!closingContentScreen
        end if 'roVideoScreenEvent
    end while
    if type(videoScreen) = "roVideoScreen" then videoScreen.Close()
End Sub